//Values for tuning
var maxSpeed = 6.0;
var acceleration = 10;
var deceleration = 10;
var rotRate = 1.0;
var rollRate = 1.0;
var smooth = 2.0;

var tailEnd : Transform;
var gameCamera : Camera;
var burstSound : AudioClip;
 
private var moveDirection = Vector3.zero;
private var rotDirection = Vector3.zero;
private var speed = 0.0;

private var bursting = false;
private var burstMax;
private var burstLock;
private var startBurstSpeed;
private var burstStartTime;

function Start() {
	burstMax = maxSpeed + maxSpeed * 0.5;
	burstLock = Time.time;		
}

function FixedUpdate() {
	
	var controller : CharacterController = GetComponent(CharacterController);
	
	//Is the user telling us to move forward? Do this check once.		
	var move = Input.GetAxis("Forward") ? true : false;
	
	// Setup the X and Y axis rotation from input
	rotDirection = new Vector3(Input.GetAxis("Vertical"), Input.GetAxis("Horizontal"), 0) * rotRate;
		
	if (bursting)
	{
		//Burst time is fixed at 3 seconds for prototyping	
		if (Time.fixedTime < burstStartTime + 3.0)
		{
			if (speed < burstMax)
			{	
				speed += acceleration * 2 * Time.deltaTime;
			}		
			// Adjust FOV
			gameCamera.fieldOfView = 60 + 10 * (Time.fixedTime - burstStartTime);
			
			// Apply rotation to the head
			for (var child : Transform in transform) 
			{
				child.transform.Rotate(Vector3.forward * 120 * Time.deltaTime);
			}
			
			//Apply Rotation to the tail
			for (var tailChild : Transform in tailEnd.transform) 
			{
				tailChild.transform.Rotate(Vector3.forward * 120 * Time.deltaTime);
			}		
		}
		else
		{
			bursting = false;
			burstLock = Time.fixedTime + 5;
		}
		
	} // If the player is not telling us to go forward, re-orient
	else if (!move)
	{
		// Keep the Y rotation so the general direction of motion is preserved
		var target = Quaternion.Euler (0, transform.eulerAngles.y, 0);
		// Dampen towards the target rotation
		transform.rotation = Quaternion.Slerp(transform.rotation, target, Time.deltaTime * smooth);
		
		// If we are still moving forward, decelerate toward zero
		if (speed > 0.0)
		{
			speed -= deceleration * Time.deltaTime;	
		}// If we dip below 0 for speed, go back to zero
		else if (speed < 0.0)
		{
			speed = 0.0;	
		}
			
	}
	else
	{
		// Check if the player wants to burst and didn't burst too recently
		if (Input.GetAxis("Burst") && burstLock < Time.fixedTime)
		{
			 bursting = true;
			 startBurstSpeed = speed;
			 burstStartTime = Time.fixedTime;
			 audio.clip = burstSound;
			 audio.Play();
		}
			
		// Roll the player left or right around the Z axis
		transform.Rotate(0,0, Mathf.Max(Input.GetAxis("RollRight"), 0.0) * -rollRate * Time.deltaTime);  	
		transform.Rotate(0,0, Mathf.Max(Input.GetAxis("RollLeft"), 0.0) * rollRate * Time.deltaTime); 
		
		if (speed <= maxSpeed)
		{
			// Accelrate
			speed += acceleration * Time.deltaTime;
		}
		
		//Pull the FOV back if we have been bursting
		if (gameCamera.fieldOfView > 60)
		{
			gameCamera.fieldOfView -= 0.2;	
		}
	}	
	
	// Apply rotation to controller
	transform.Rotate(rotDirection * Time.deltaTime);
	
	//Create the move vector
	moveDirection = new Vector3(0, 0, speed);
	
	//Convert movement from local to world axis
	var worldMove = transform.TransformDirection(moveDirection);
	//Apply the translation using controller move function to check for collision
	var flags = controller.Move(worldMove * Time.deltaTime);
}