// All the segments of the tail, minus the end
var tail:Array;
// The prefab to use when creating new segments
var segment:GameObject;
// The tail end to keep at the back
var tailEnd:GameObject;
// The music to fade out at the end
var music : AudioSource;

// The texture to use during the ending fade
var fadeOutTexture : Texture2D;

static var tailLength = 0;
static var collected = 0;

// Alpha for end fade
private var alpha = 0.0;
// True when win condition is reached and we should fade out
private var end = false;

function Start() {
	tail = new Array();
}

function Update() {
	
	// Check if enough rings are collected to add a new segment
	if (tail.length < Mathf.FloorToInt(collected / 10))
	{
		AddSegment();	
	}	
	
	//Start with the head transform
	var previousTarget = transform;
	
	//Have each segment follow the previous one, updating in order
	for (tailSegment in tail)
	{
		tailSegment.GetComponent(TailFollow).FollowTarget(previousTarget);
		previousTarget = tailSegment.transform;	
	}
	
	//Finally update the end
	tailEnd.GetComponent(TailFollow).FollowTarget(previousTarget);	
		
	//Test if the escape key got hit, for quiting on Windows
	if (Input.GetKey ("escape")) 
	{
		end = true;
	}
}

function AddSegment() {
	//Create a new instance of the template
	var newTail = Instantiate(segment, Vector3.zero, Quaternion.identity);
	
	//Give it a logical name for debugging
	newTail.name = "Tail" + tail.length;
	
	var target : Transform;

	//If we are the first segment we need to follow the player head	
	if (tail.length == 0)
	{
		target = transform;
	}
	else
	{
		target = tail[tail.length - 1].transform;	
	}
		
	//Sync the new segment with its target & target transform
	newTail.transform.rotation = target.rotation;
	newTail.transform.position = target.transform.position;	
	
	tail.Add(newTail);
	tailLength++;
	
	//Check if the end condition has been met
	if (tailLength >= 7)
	{
		end = true;	
	}
}

function OnGUI() {

	// If win condition is met, fade out the game and quit.	
	if (end)
	{
	    alpha += 0.1 * Time.deltaTime;   
	    
	    GUI.color.a = alpha;
	   
	    GUI.depth = -1000;
	   
	    GUI.DrawTexture(Rect(0, 0, Screen.width, Screen.height), fadeOutTexture);
	    
	    if (music.volume > 0.0)
	    {
	    	music.volume -= 0.01 * Time.deltaTime;	
	    }
	    
	    if (alpha >= 1.0)
	    {
	    	Application.Quit();	
	    }
	}
}
